/*
 * Copyright (c) 2017, Red Hat Inc. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.shenandoah.wip;

import org.openjdk.jmh.annotations.*;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.TimeUnit;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Threads(1)
@State(Scope.Benchmark)
public class WeakRefs {

    @Param({"1", "10", "100", "1000", "10000"})
    int count;

    ReferenceQueue<Target> refq;
    Target[] targets;
    MyRef[] refs;

    @Setup
    public void setup() {
        refq = new ReferenceQueue<>();
        targets = new Target[count];
        refs = new MyRef[count];
        for (int c = 0; c < count; c++) {
            Target o = new Target();
            targets[c] = o;
            refs[c] = new MyRef(refq, o, c);
        }
    }

    @Benchmark
    public void test() {
        targets[ThreadLocalRandom.current().nextInt(count)] = null;

        MyRef r;
        while ((r = (MyRef)refq.poll()) != null) {
            Target o = new Target();
            targets[r.index] = o;
            refs[r.index] = new MyRef(refq, o, r.index);
        }
    }

    static class MyRef extends WeakReference<Target> {
        int index;
        public MyRef(ReferenceQueue<Target> refq, Target obj, int idx) {
            super(obj, refq);
            this.index = idx;
        }
    }

    static class Target {

    }

}
